;   Name of file: LOG.LSP
;
;   LOG.LSP creates a new AutoCAD command called LOG. When invoked, LOG  
;   prompts for the user's name, then redefines AutoCAD's END command so 
;   that in addition to ending the drawing session, END will calculate the
;   time spent editing the drawing and append that information to a file 
;   on the hard disk. LOG is intended to be invoked at the beginning of
;   each drawing session. LOG.LSP can be added to ACAD.LSP.
;
;   NOTE: The Getstring T function, as well as the Command function calling
;         UNDEFINE, don't seem to work well in ACAD.LSP unless nested 
;         inside Defun, as below. Once LOG.LSP is added to ACAD.LSP, 
;         you can make LOG.LSP fully automatic by calling the LOG 
;         command from a script file when loading AutoCAD. (See ACAD docs 
;         pgs. 269-270 re: script files when loading ACAD)
;
;   In the routine below, the new END command creates and adds to a file 
;   called "diary" (line 77). Note that forward slashes are used 
;   in LISP routines to indicate the directory path. This path/file name 
;   could be easily changed to whatever path/file name you choose.
;
;   This routine requires AutoCAD Release 9.
;
;   Written by: Bob Thomas, Thomas Enterprises, ltd.
;                           628 Clement Street
;                           San Francisco, CA  94118
;                           (415) 751-3275
;   Last changed:  10/09/87
;
;
(setq stime 
      (getvar "TDINDWG") 
) 
(defun C:LOG() 
  (setq oldcmd (getvar "CMDECHO")) 
  (setvar "CMDECHO" 0) 
  (if (/= uname nil) 
      (progn 
          (setq temp uname) 
          (setq uname
                (getstring t (strcat "\nYour Name? <" uname ">: "))
          )
          (if (or
                (= uname nil)
                (= uname "")
              )
              (setq uname temp)
          )
      )        
  )
  (while (or
            (= uname nil)
            (= uname "")
         )
         (setq uname
               (getstring t "\nYour Name? ")
         )
  )
  (command "UNDEFINE"
         "END"
  )
  (defun C:END()
    (setq uname (strcat uname "                         "))
    (setq x (getvar "DWGNAME"))
    (setq etime 
          (* (- (getvar "TDINDWG") stime) 24.0)
    )
    (setq month
        (substr (rtos (getvar "CDATE") 2 0) 5 2)
    )
    (setq day 
        (substr (rtos (getvar "CDATE") 2 0) 7 2)
    )
    (setq year 
        (substr (rtos (getvar "CDATE") 2 0) 3 2)
    )
    (setq diary (open "diary" "a"))   ; This file name can be
                                      ; changed to whatever file/path 
                                      ; you prefer
    (write-line (strcat "User: " 
                        (substr uname 1 25) 
                        " -- Date: " 
                         month "/" day "/" year
                ) 
                diary
    )
    (write-line (strcat "Drawing: " x) diary)
    (write-line (strcat "Editing time: " 
                         (itoa (fix etime))
                        " hours, "
                        (itoa (fix (setq mins
                                         (* (- etime (fix etime)) 60.0)
                                   )
                              )
                        )
                        " minutes, "
                        (itoa (fix (* (- mins (fix mins)) 60.0)))
                        " seconds."
                )
                diary
    )
    (write-line "* * * * * * * * * * * * * * * * * * * * * *" diary)
    (close diary)
    (command ".END")
  )
  (setvar "CMDECHO" oldcmd)
  (princ)
)
;
;
;  END OF FILE:    LOG.LSP    111 LINES.  Bob Thomas

